@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 4084 2008-08-04 21:19:09Z wwalker $"
-__version__   = "$Revision: 4084 $"
-__date__      = "$Date: 2008-08-04 17:19:09 -0400 (Mon, 04 Aug 2008) $"
+__id__        = "$Id: braillegenerator.py 4417 2009-01-09 16:33:34Z wwalker $"
+__version__   = "$Revision: 4417 $"
+__date__      = "$Date: 2009-01-09 11:33:34 -0500 (Fri, 09 Jan 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1085,7 +1085,8 @@ class BrailleGenerator:
         componentRegion = braille.Component(
             obj, text,
             indicator=settings.brailleRadioButtonIndicators[ \
-            int(state.contains(pyatspi.STATE_CHECKED))])
+            int(state.contains(pyatspi.STATE_CHECKED) \
+                or state.contains(pyatspi.STATE_PRESSED))])
         regions.append(componentRegion)
 
         return [regions, componentRegion]
@@ -1732,6 +1733,9 @@ class BrailleGenerator:
         brailleRolenameStyle = settings.brailleRolenameStyle
 
         regions = []
+        if not settings.enableBrailleContext:
+            return regions
+
         parent = obj.parent
         if parent and (parent.getRole() in self.SKIP_CONTEXT_ROLES):
             parent = parent.parent
