@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 4191 2008-09-11 17:30:11Z joanied $"
-__version__   = "$Revision: 4191 $"
-__date__      = "$Date: 2008-09-11 13:30:11 -0400 (Thu, 11 Sep 2008) $"
+__id__        = "$Id: flat_review.py 4413 2009-01-08 17:31:58Z wwalker $"
+__version__   = "$Revision: 4413 $"
+__date__      = "$Date: 2009-01-08 12:31:58 -0500 (Thu, 08 Jan 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -135,9 +135,14 @@ class Word:
                     [char, startOffset, endOffset] = text.getTextAtOffset(
                         self.startOffset + i,
                         pyatspi.TEXT_BOUNDARY_CHAR)
+                    # Sometimes we get more than a character's worth. See
+                    # Bug #495303. We can try to correct this.
+                    #
+                    if len(char):
+                        char = char.decode("UTF-8")[0].encode("UTF-8")
                     [x, y, width, height] = text.getRangeExtents(
                         startOffset,
-                        endOffset,
+                        startOffset + 1,
                         0)
                     self.chars.append(Char(self,
                                            i,
@@ -638,7 +643,7 @@ class Context:
             currentZoneIndex = 0
             while currentZoneIndex < len(line.zones):
                 zone = line.zones[currentZoneIndex]
-                if zone.accessible == searchZone:
+                if self.script.isSameObject(zone.accessible, searchZone):
                     foundZoneWithFocus = True
                     break
                 else:
@@ -854,10 +859,45 @@ class Context:
         else:
             zones = []
 
-        length = text.characterCount
-
         offset = 0
         lastEndOffset = -1
+        upperMax = lowerMax = text.characterCount
+        upperMid = lowerMid = upperMax /2
+        upperMin = lowerMin = 0
+        upperY = lowerY = 0
+        oldMid = 0
+
+        # performing binary search to locate first line inside clipped area
+        while oldMid != upperMid:
+            oldMid = upperMid
+            [x, y, width, height] = text.getRangeExtents(upperMid,
+                                                         upperMid+1,
+                                                         0)
+            upperY = y
+            if y > cliprect.y:
+                upperMax = upperMid
+            else:
+                upperMin = upperMid
+            upperMid = (upperMax - upperMin) /2 + upperMin
+
+        # performing binary search to locate last line inside clipped area
+        oldMid = 0
+        limit = cliprect.y+cliprect.height
+        while oldMid != lowerMid:
+            oldMid = lowerMid
+            [x, y, width, height] = text.getRangeExtents(lowerMid,
+                                                         lowerMid+1,
+                                                         0)
+            lowerY = y
+            if y > limit:
+                lowerMax = lowerMid
+            else:
+                lowerMin = lowerMid
+            lowerMid = (lowerMax - lowerMin) /2 + lowerMin
+
+        # finding out the zones
+        offset = upperMin
+        length = lowerMax
         while offset < length:
 
             [string, startOffset, endOffset] = text.getTextAtOffset(
