@@ -19,9 +19,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id$"
-__version__   = "$Revision$"
-__date__      = "$Date$"
+__id__        = "$Id: speech_generator.py 4372 2008-12-01 18:49:20Z joanied $"
+__version__   = "$Revision: 4372 $"
+__date__      = "$Date: 2008-12-01 13:49:20 -0500 (Mon, 01 Dec 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -280,7 +280,17 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
             if script_settings.speakCellCoordinates:
                 nameList = obj.name.split()
-                utterances.append(nameList[1])
+                # We were assuming that the word for "cell" would always
+                # precede the coordinates. This is not the case for all
+                # languages (e.g. Hungarian). See bug #562532. Therefore
+                # examine each item and choose the one which contains a
+                # digit.
+                #
+                for name in nameList:                    
+                    for char in name.decode("UTF-8"):
+                        if char.isdigit():
+                            utterances.append(name)
+                            return utterances
         else:
             # Check to see how many children this table cell has. If it's
             # just one (or none), then pass it on to the superclass to be
