@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 4161 2008-09-06 18:30:21Z joanied $"
-__version__   = "$Revision: 4161 $"
-__date__      = "$Date: 2008-09-06 14:30:21 -0400 (Sat, 06 Sep 2008) $"
+__id__        = "$Id: speechgenerator.py 4417 2009-01-09 16:33:34Z wwalker $"
+__version__   = "$Revision: 4417 $"
+__date__      = "$Date: 2009-01-09 11:33:34 -0500 (Fri, 09 Jan 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1702,7 +1702,8 @@ class SpeechGenerator:
 
         utterances = []
         state = obj.getState()
-        if state.contains(pyatspi.STATE_CHECKED):
+        if state.contains(pyatspi.STATE_CHECKED) \
+           or state.contains(pyatspi.STATE_PRESSED):
             # Translators: the state of a toggle button.
             #
             checkedState = _("pressed")
